package org.playframework.playtools.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.internal.ui.packageview.ClassPathContainer;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.playframework.playtools.PlayToolsPlugin;
import org.playframework.playtools.preferences.PreferenceConstants;

public class PlayToolsUtil {

	public static final int PLAY_VERSION_NONE = 0;
	public static final int PLAY_VERSION_1 = 1;
	public static final int PLAY_VERSION_2 = 2;

	public static final int PLAY_PROJECT_APPLICATION_EMPTY = 0;
	public static final int PLAY_PROJECT_APPLICATION_JAVA = 1;
	public static final int PLAY_PROJECT_APPLICATION_SCALA = 2;
	
	public static final int PLAY_START_OK = 0;
	public static final int PLAY_ROOT_PAT_NONE = 1;
	public static final int PLAY_SELECT_PROJECT_NONE = 2;

	public static final String PLAY_VERSION1_SERVER_PORT = "8000";
	public static final String PLAY_VERSION2_SERVER_PORT = "9999";
	
	public static final String application_items[] = { "empty project", "Java application", "Scala application"};
	
	private static final String PLAY_ECLIPSE_VERTION = "Play Framework Version=";

	
	private static final String LINE_SPLIT = "\n";
	private static final String ITEM_SPLIT = "\t";

	private static String selectProjectPath = null;
	
	public static List<String[]> paserLocation(String locations) {
        List<String[]> list = new ArrayList<String[]>();

        if (locations.length() > 0) {
			String[] values = locations.split(LINE_SPLIT);
	        for (String val : values) {
	        	String[] items = val.split(ITEM_SPLIT);
	        	list.add(items);
	        }
        }
		return list;
	}

	public static String[] paserLocationVersion(String locations) {
        List<String> list = new ArrayList<String>();

        if (locations.length() > 0) {
			String[] values = locations.split(LINE_SPLIT);
	        for (String val : values) {
	        	String[] items = val.split(ITEM_SPLIT);
	        	list.add(items[0]);
	        }
        }
		return (String[])list.toArray(new String[list.size()]);
	}

	public static String wordsToString(List<String[]> list) {
        StringBuffer stringBuffer = new StringBuffer();

        for (String[] items : list) {
        	stringBuffer.append(items[0]);
        	stringBuffer.append(ITEM_SPLIT);
        	stringBuffer.append(items[1]);
        	stringBuffer.append(ITEM_SPLIT);
        	stringBuffer.append(items[2]);
        	stringBuffer.append(LINE_SPLIT);
        }
		return stringBuffer.toString();
	}

	public static int getPlayFrameworkVersionID(String version) {
		int result = PLAY_VERSION_NONE;
		if (version.length() == 0) {
			return result;
		}
		if (version.substring(0,1).equals("1")) {
			result = PLAY_VERSION_1;
		} else if(version.substring(0,1).equals("2")) {
			result = PLAY_VERSION_2;
		}
		return result;
	}

	public static String searchPlayVersion(String path) {
		if (path == null || path.length() == 0) {
			return "";
		}
        String filename = path + "/framework/src/play/version";
        String version = readFile(filename, false);

        if (version.length() > 0) {
        	return version;
        }
        
        return getVersion2(path + "/framework/build");
	}
	
    public static String getVersion2(String filename) {
    	String version = "";
        String vers = readFile(filename, true);

        String[] arrayVar = vers.split("\n");
        for (int i = 0; i < arrayVar.length; i++) {
        	if (arrayVar[i].indexOf("PLAY_VERSION") >= 0) {
        		String buf = arrayVar[i].replace("PLAY_VERSION", "");
        		buf = buf.replace("\"", "");
        		buf = buf.replace("=", "");
        		version = buf.trim();
        		break;
        	}
        }
    	return version;
    }
	
	public static String readFile(String filename, boolean lfcode) {
		String lf = "";
		if (lfcode) {
			lf = "\n";
		}
		StringBuffer stringBuffer = new StringBuffer();
		try{
			File file = new File(filename);
			if (file.exists()) {
				FileReader fileReader = new FileReader(filename);
				BufferedReader reader = new BufferedReader(fileReader);
				String line;
				while((line = reader.readLine())!=null){
					stringBuffer.append(line + lf);
				}
				reader.close();
				fileReader.close();
			}
		} catch(Exception e){
			e.printStackTrace();
		}
		return stringBuffer.toString();
	}

	public static boolean isPlayProject(String projectPath) {
        String filename = projectPath + "/version";
        String versionFile = readFile(filename, false);

        if (versionFile.indexOf(PLAY_ECLIPSE_VERTION) >= 0) {
            return true;
        }
        return false;
	}

	public static String getPlayVersion(String projectPath) {
        String filename = projectPath + "/version";
        String versionFile = readFile(filename, false);

        if (versionFile.indexOf(PLAY_ECLIPSE_VERTION) >= 0) {
        	return versionFile.replace(PLAY_ECLIPSE_VERTION, "").trim();
        }
        return "";
    }

	public static IProject getProjectHandle(String projectName) {
        return ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
    }
	
	public static String getPlayRootPath(String version) {
		String locations = PlayToolsPlugin.getDefault()
        		.getPreferenceStore().getString(PreferenceConstants.P_PATH_PLAY);
        List<String[]> listArray = PlayToolsUtil.paserLocation(locations);
		
        for (String[] item : listArray) {
        	if (version.equals(item[0])) {
        		return item[1].replace("\\", "/");
        	}
        }
        return "";
	}

	public static String getProjectPlayRootPath(String projectPath) {
		String version = getPlayVersion(projectPath);
		return getPlayRootPath(version);
	}

	public static String getServerPort(String version) {
		String locations = PlayToolsPlugin.getDefault()
        		.getPreferenceStore().getString(PreferenceConstants.P_PATH_PLAY);
        List<String[]> listArray = PlayToolsUtil.paserLocation(locations);
		
        for (String[] item : listArray) {
        	if (version.equals(item[0])) {
        		return item[2];
        	}
        }
        return "";
	}

    public static String getSelectProjectPath() {
    	selectProjectPath = "";
    	
		try {
	    	IWorkbench iworkbench = PlatformUI.getWorkbench();
			if (iworkbench == null) {
				return "";
			}
			IWorkbenchWindow iworkbenchwindow = iworkbench.getActiveWorkbenchWindow();
			if (iworkbenchwindow == null) {
				return "";
			}

			IWorkbenchPage iworkbenchpage = iworkbenchwindow.getActivePage();
			if (iworkbenchpage == null) {
				return "";
			}

			ISelection selection = iworkbenchpage.getSelection();
			if (selection == null) {
				return "";
			}

			if (selection instanceof IStructuredSelection) {
				IStructuredSelection sSelection = (IStructuredSelection) selection;
				for (Iterator<?> iterator = sSelection.iterator(); iterator.hasNext();) {
					Object type = (Object) iterator.next();
					if (type instanceof IResource) {
						IContainer container;
						if (type instanceof IContainer) {
							container = (IContainer) type;
						} else {
							container = ((IResource) type).getParent();
						}
						selectProjectPath = container.getProject().getLocation().toOSString();
						break;
					} else if (type instanceof IJavaElement) {
						IJavaElement element = (IJavaElement) type;
						IResource resource = element.getResource();
						if (resource == null) {
							resource = element.getParent().getResource();
						}
						selectProjectPath = resource.getProject().getLocation().toOSString();
						break;
					} else if (type instanceof ClassPathContainer) {
						ClassPathContainer container = (ClassPathContainer) type;
						selectProjectPath = container.getJavaProject().getProject().getLocation().toOSString();
						break;
					}
				}
			} else if (selection instanceof ITextSelection) {
			    IEditorPart editor = iworkbenchpage.getActiveEditor();
			    if (editor != null) {
				    IJavaElement element = JavaUI.getEditorInputJavaElement(editor.getEditorInput());

					IResource resource = element.getResource();
					selectProjectPath = resource.getProject().getLocation().toOSString();
			    }
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
		if (selectProjectPath == null || "".equals(selectProjectPath)) {
			return "";
		}
    	return selectProjectPath.replace("\\", "/");
    }

	public static int isStart() {
		String path = getSelectProjectPath();
		if ("".equals(path)) {
        	return PLAY_SELECT_PROJECT_NONE;
		}
		String playPath = getProjectPlayRootPath(path);
        if ("".equals(playPath)) {
        	return PLAY_ROOT_PAT_NONE;
        }
		return PLAY_START_OK;
	}

	public static List<String> getClasspath(String frameworkPath) {
		List<String> listClass = new ArrayList<String>();
		
    	String path = frameworkPath + "/framework";
        File dir = new File(path);
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            String file = files[i].toString().replace("\\", "/");
            String ext = getExt(file);
            if (".jar".equals(ext)) {
                listClass.add(file);
            }
        }

    	path += "/lib";
        dir = new File(path);
        files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            String file = files[i].toString().replace("\\", "/");
            String ext = getExt(file);
            if (".jar".equals(ext)) {
                listClass.add(file);
            }
        }
        return listClass;
    }
	
	public static String getExt(String filename) {
    	String ext = "";
    	int inx = filename.lastIndexOf(".");
    	if (inx > 0) {
    		ext = filename.substring(inx);
    	}
    	return ext;
    }

}
