package org.playframework.playtools.util.config;


import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.playframework.playtools.util.PlayToolsUtil;

public abstract class AbstractPlayConfigManager implements IPlayConfigManager {

	protected static final String PLAY_ECLIPSE_VERTION = "Play Framework Version=";

	protected static final String FILE_APPLICATION = "application.conf";
	protected static final String FILE_PROJECT = ".project";
	protected static final String FILE_CLASSPATH = ".classpath";
	protected static final String FILE_VERSION = "version";
	protected static final String FILE_CONNECT_LAUNCH = "connect.launch";
	protected static final String FILE_TEST_LAUNCH = "test.launch";
	protected static final String FILE_DEBUG_LAUNCH = "debug.launch";


	protected static String workspacePath = null;
	protected static String projectName = null;
	protected static String projectPath = null;
	protected static String selectProjectPath = null;
	protected String version = null;
	protected int    aplication = -1;
	protected String payRootPath = null;
	
    public IProject getProjectHandle(String projectName) {
        return ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
    }

    public String getPlayRootPath() {
    	return payRootPath;
    }
    public String getPlayRootPathTrim() {
    	String playPath = payRootPath;
    	playPath = playPath.replace("\\", "/");
    	if (playPath.length() > 2 && "/".equals(playPath.substring(playPath.length()-1))) {
    		playPath = playPath.substring(0, playPath.length()-1);
    	}
    	return playPath;
    }

    public void setWorkspacePath(String path) {
    	path = path.replace("\\", "/");
    	if (path.length() > 2 && "/".equals(path.substring(path.length()-1))) {
    		path = path.substring(0, path.length()-1);
    	}
    	workspacePath = path;
    }

    public String getWorkspacePath() {
    	return workspacePath;
    }
    public void setProjectName(String name) {
    	projectName = name;
    }
    public String getProjectName() {
    	return projectName;
    }
    public void setProjectPath(String path) {
    	projectPath = path;
    }
    public String getProjectPath() {
    	return projectPath;
    }

    protected void createFolder(IProgressMonitor monitor,
			IContainer container, String pathName) {
		IPath path = new Path(pathName);
		IFolder folder = container.getFolder(path);
		if (!folder.exists()) {
			try {
				folder.create(true, true, monitor);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}

    protected void createRootToStream(IProject projectHandle, String srcPath, String filename) {
		String buf = readFile(srcPath + "/" + filename);

		createRootFile(projectHandle, filename, buf);

	}
	protected void createRootFile(IProject projectHandle,
					String fileName, String streamData) {
		if (projectHandle.exists()) {
			IFile newFile = projectHandle.getFile(fileName);
			InputStream fileStream = new ByteArrayInputStream(streamData.getBytes());;
			try {
				if (newFile.exists()) {
					newFile.setContents(fileStream, true, true, null);
				} else {
					newFile.create(fileStream, true, null);
				}
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}

	protected String readFile(String filename, boolean lfcode) {
		return PlayToolsUtil.readFile(filename, lfcode);
	}
	
	protected String readFile(String filename) {
		return readFile(filename, true);
	}
	
	protected void createFile(IProject projectHandle,
			String destPath, String fileName, String streamData) {
		IFolder controllersFolder = projectHandle.getFolder(destPath);
		if (controllersFolder.exists()) {
			IFile newFile = controllersFolder.getFile(fileName);
			InputStream fileStream = new ByteArrayInputStream(streamData.getBytes());;
			try {
				newFile.create(fileStream, false, null);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}
	
	protected void createFileToStream(IProject projectHandle,
			String srcPath, String destPath, String fileName) {
		IFolder controllersFolder = projectHandle.getFolder(destPath);
		if (controllersFolder.exists()) {
			IFile newFile = controllersFolder.getFile(fileName);
			FileInputStream fileStream;
			try {
				fileStream = new FileInputStream(srcPath + destPath + "/" + fileName);
				newFile.create(fileStream, false, null);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}

	protected String getPlayVersion() {
		return version;
	}
}
