package org.playframework.playtools.util.config;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.IProgressMonitor;
import org.playframework.playtools.util.PlayToolsUtil;

public class PlayConfigManager {

	protected static IPlayConfigManager configManager = null;
	
	private static String version;
	private static int application;
	private static String payRootPath;
	
	public static IPlayConfigManager PlayConfigManagerFactory(String ver, int app) {
		version = ver;
		application = app;
		configManager = null;
		
		payRootPath = PlayToolsUtil.getPlayRootPath(version);
		int id = PlayToolsUtil.getPlayFrameworkVersionID(version);
		
		if (id == PlayToolsUtil.PLAY_VERSION_1) {
			configManager = new PlayConfigManagerVersion1(version, application, payRootPath);
		} else if (id == PlayToolsUtil.PLAY_VERSION_2) {
			configManager = new PlayConfigManagerVersion2(version, application, payRootPath);
		}
		return configManager;
	}

	public static void createPlayProject(IProject projectHandle,
			IProjectDescription description, IProgressMonitor monitor) {
		if (configManager != null) {
			configManager.createPlayProject(projectHandle, description, monitor);
		}
	}

	public static void createPlayProjectFile() {
		if (configManager != null) {
			configManager.createPlayProjectFile();
		}
	}

	public static String getPlayRootPath() {
		if (configManager != null) {
			return configManager.getPlayRootPath();
		}
		return "";
	}

	public static String getPlayRootPathTrim() {
		if (configManager != null) {
			return configManager.getPlayRootPathTrim();
		}
		return "";
	}

	public static IProject getProjectHandle(String projectName) {
		if (configManager != null) {
			return configManager.getProjectHandle(projectName);
		}
		return null;
	}

	public static String getProjectName() {
		if (configManager != null) {
			return configManager.getProjectName();
		}
		return "";
	}

	public static String getProjectPath() {
		if (configManager != null) {
			return configManager.getProjectPath();
		}
		return "";
	}

	public static String getWorkspacePath() {
		if (configManager != null) {
			return configManager.getWorkspacePath();
		}
		return "";
	}

	public static void setProjectName(String name) {
		if (configManager != null) {
			configManager.setProjectName(name);
		}
	}

	public static void setProjectPath(String path) {
		if (configManager != null) {
			configManager.setProjectPath(path);
		}
	}

	public static void setWorkspacePath(String path) {
		if (configManager != null) {
			configManager.setWorkspacePath(path);
		}
	}

}
