package org.playframework.playtools.util.config;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;


public class PlayConfigManagerVersion1 extends AbstractPlayConfigManager {

	private static final String RANDOMCHOICE = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
	private static final String JPDA_PORT = "8000";

	private static final String PATH_SETTING = "/.settings";
	private static final String PATH_APP = "/app";
	private static final String PATH_CONTROLLERS = PATH_APP + "/controllers";
	private static final String PATH_MODELS = PATH_APP + "/models";
	private static final String PATH_VIEWS = PATH_APP + "/views";
	private static final String PATH_APPLICATION = PATH_VIEWS + "/Application";
	private static final String PATH_ERRORS = PATH_VIEWS + "/errors";
	private static final String PATH_CONF = "/conf";
	private static final String PATH_ECLIPSE = "/eclipse";
	private static final String PATH_LIB = "/lib";
	private static final String PATH_PUBLIC = "/public";
	private static final String PATH_IMAGES = PATH_PUBLIC + "/images";
	private static final String PATH_JAVASCRIPTS = PATH_PUBLIC + "/javascripts";
	private static final String PATH_STYLESHEETS = PATH_PUBLIC + "/stylesheets";
	private static final String PATH_TEST = "/test";
	private static final String PATH_RESOURCE = "/resources";
	private static final String PATH_RESOURCE_SKEL = PATH_RESOURCE + "/application-skel";
	private static final String PATH_RESOURCE_ECLIPSE = PATH_RESOURCE + "/eclipse";
	private static final String PATH_FRAMEWORK = "/framework";
	private static final String PATH_FRAMEWORK_LIB = PATH_FRAMEWORK + PATH_LIB;
	private static final String PATH_SRC = PATH_FRAMEWORK + "/src";

	public PlayConfigManagerVersion1() {
	}
	
	public PlayConfigManagerVersion1(String version, int aplication, String payRootPath) {
		this.version = version;
		this.aplication = aplication;
		this.payRootPath = payRootPath;
	}

    public void createPlayProject(IProject projectHandle,
    		IProjectDescription description, IProgressMonitor monitor) {

    	String playPath = getPlayRootPathTrim();

		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(getProjectName()));
		IContainer container = (IContainer) resource;

		createFolder(monitor, container, PATH_SETTING);
		createFolder(monitor, container, PATH_APP);
		createFolder(monitor, container, PATH_CONTROLLERS);
		createFolder(monitor, container, PATH_MODELS);
		createFolder(monitor, container, PATH_VIEWS);

		createFolder(monitor, container, PATH_APPLICATION);
		createFolder(monitor, container, PATH_VIEWS);

		createFolder(monitor, container, PATH_CONF);
		createFolder(monitor, container, PATH_ECLIPSE);
		createFolder(monitor, container, PATH_LIB);
		createFolder(monitor, container, PATH_PUBLIC);
		createFolder(monitor, container, PATH_IMAGES);
		createFolder(monitor, container, PATH_JAVASCRIPTS);
		createFolder(monitor, container, PATH_STYLESHEETS);
		createFolder(monitor, container, PATH_TEST);

		String applicationPath = playPath + PATH_RESOURCE_SKEL;
		String eclipsePath = playPath + PATH_RESOURCE_ECLIPSE;

        createRootFile(projectHandle, FILE_VERSION, PLAY_ECLIPSE_VERTION + getPlayVersion());

		createFileToStream(projectHandle, applicationPath, PATH_CONTROLLERS, "Application.java");

        createFileToStream(projectHandle, applicationPath, PATH_VIEWS, "main.html");
        createFileToStream(projectHandle, applicationPath, PATH_APPLICATION, "index.html");
        createFileToStream(projectHandle, applicationPath, PATH_ERRORS, "404.html");
        createFileToStream(projectHandle, applicationPath, PATH_ERRORS, "500.html");

        createFileToStream(projectHandle, applicationPath, PATH_CONF, "dependencies.yml");
        createFileToStream(projectHandle, applicationPath, PATH_CONF, "routes");
        createFileToStream(projectHandle, applicationPath, PATH_CONF, "messages");

        createFileToStream(projectHandle, applicationPath, PATH_IMAGES, "favicon.png");
        createFileToStream(projectHandle, applicationPath, PATH_JAVASCRIPTS, "jquery-1.6.4.min.js");
        createFileToStream(projectHandle, applicationPath, PATH_STYLESHEETS, "main.css");

        createFileToStream(projectHandle, applicationPath, PATH_TEST, "data.yml");
        createFileToStream(projectHandle, applicationPath, PATH_TEST, "BasicTest.java");
        createFileToStream(projectHandle, applicationPath, PATH_TEST, "ApplicationTest.java");
        createFileToStream(projectHandle, applicationPath, PATH_TEST, "Application.test.html");

        createFileToStream(projectHandle, eclipsePath, PATH_SETTING, "org.eclipse.jdt.core.prefs");
        createFileToStream(projectHandle, eclipsePath, PATH_SETTING, "org.eclipse.core.resources.prefs");

        createApplicationConfig(projectHandle, applicationPath, PATH_CONF);

        createLaunch(projectHandle, eclipsePath, PATH_ECLIPSE, playPath);

        createClasspath(projectHandle);
    }

    private void createApplicationConfig(IProject projectHandle,
    										String srcPath, String destPath) {

    	String file = readFile(srcPath + destPath + "/" + FILE_APPLICATION);

        String key = secretKey();

    	String buf = file.replace("%APPLICATION_NAME%", getProjectName());

    	createFile(projectHandle, destPath, FILE_APPLICATION, buf.replace("%SECRET_KEY%", key));
    }

    private void createLaunch(IProject projectHandle, String srcPath, String destPath, String playPath) {
    	String varsion = getPlayVersion();

    	String lanchData = createLancheData(srcPath, FILE_CONNECT_LAUNCH, playPath, varsion);

    	createFile(projectHandle, destPath, "Connect JPDA to " + getProjectName() + ".launch", lanchData);

    	lanchData = createLancheData(srcPath, FILE_TEST_LAUNCH, playPath, varsion);
    	createFile(projectHandle, destPath, "Test " + getProjectName() + ".launch", lanchData);

    	lanchData = createLancheData(srcPath, FILE_DEBUG_LAUNCH, playPath, varsion);
    	createFile(projectHandle, destPath, getProjectName() + ".launch", lanchData);
    }

    private String createLancheData(String srcPath,
    						String fileName, String playPath, String version) {
    	String file = readFile(srcPath + "/" + fileName);

    	file = file.replace("%PROJECT_NAME%", projectName);
    	file = file.replace("%PLAY_BASE%", playPath);
    	file = file.replace("%PLAY_VERSION%", version);

    	file = file.replace("%JPDA_PORT%", JPDA_PORT);
    	file = file.replace("%PLAY_ID%", " ");

    	return file;
    }

    private static String secretKey() {
	    Random rnd = new Random();

	    StringBuffer stringBuffer = new StringBuffer();
	    for (int i = 0; i < 64; i++) {
	    	int ran = rnd.nextInt(62);

	    	stringBuffer.append(RANDOMCHOICE.substring(ran, ran + 1));
	    }
	    return stringBuffer.toString();
    }

    public void createPlayProjectFile() {
    	String playPath = getPlayRootPathTrim();

        createProjectConfig(playPath + PATH_RESOURCE_ECLIPSE);
    }

    public String getPlayVersion() {
    	return version;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
	public void getClasspath(List listClass, List listDocurl) {
    	String path = getPlayRootPathTrim() + PATH_FRAMEWORK;
        File dir = new File(path);
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            String file = files[i].toString().replace("\\", "/");
            String ext = getExt(file);
            if (".jar".equals(ext)) {
                listClass.add(file);
            }
        }

    	path = getPlayRootPathTrim() + PATH_FRAMEWORK_LIB;
        dir = new File(path);
        files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            String file = files[i].toString().replace("\\", "/");
            String ext = getExt(file);
            if (".jar".equals(ext)) {
                listClass.add(file);
            } else if(".docurl".equals(ext)) {
            	listDocurl.add(file);
            }
        }
    }

    @SuppressWarnings("rawtypes")
	public void createClasspath(IProject projectHandle) {
		List listClass = new ArrayList();
    	List listDocurl = new ArrayList();

    	StringBuffer stringBuffer = new StringBuffer();

    	getClasspath(listClass, listDocurl);

    	for (int i = 0; i < listClass.size(); i++) {
    		String jarFile = listClass.get(i).toString();

    		if (isPaly(jarFile)) {
    			stringBuffer.append("<classpathentry kind=\"lib\" path=\""
    				+ jarFile + "\" sourcepath=\"" + getPlayRootPathTrim() + PATH_SRC + "\" />\n");

    		} else {
                String docurlFile = jarFile.replace(".jar", ".docurl");
    			if(listDocurl.contains(docurlFile)) {
    				String docurl = readFile(docurlFile, false);

    				stringBuffer.append("\t<classpathentry kind=\"lib\" path=\"" + jarFile + "\">\n");
    				stringBuffer.append("\t\t<attributes>\n");
    				stringBuffer.append("\t\t\t<attribute name=\"javadoc_location\" value=\"" + docurl + "\"/>\n");
    				stringBuffer.append("\t\t</attributes>\n");
    				stringBuffer.append("\t</classpathentry>\n");
    			} else {
    				stringBuffer.append("\t<classpathentry kind=\"lib\" path=\"" + jarFile + "\"/>\n");
    			}

    		}
    	}
    	String file = readFile(getPlayRootPathTrim() + PATH_RESOURCE_ECLIPSE + "/"+ FILE_CLASSPATH);

    	String streamData = file.replace("%TESTCLASSPATH%", "");
    	streamData = streamData.replace("%MODULES%", "");
    	streamData = streamData.replace("%PROJECTCLASSPATH%", stringBuffer.toString());

    	createRootFile(projectHandle, ".classpath", streamData);
    }

	private void createProjectConfig(String srcPath) {
		String file = readFile(srcPath + "/" + FILE_PROJECT);

		String buf = file.replace("%PROJECT_NAME%", getProjectName());
		buf = buf.replace("%LINKS%", "");

		String filename = getWorkspacePath() + "/";

		String project = getProjectPath();

		if (!"".equals(project)) {
			filename += (project + "/");
		}
		filename += FILE_PROJECT;

		FileWriter fileWr;
		try {
			fileWr = new FileWriter(filename);
			fileWr.write(buf);
			fileWr.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

    private boolean isPaly(String filename) {
    	int inx = filename.lastIndexOf("/");
    	String file = filename.substring(inx+1);
    	String playfile = "play-" + getPlayVersion() + ".jar";
    	if (playfile.equals(file)) {
    		return true;
    	}
    	return false;
    }

    private String getExt(String filename) {
    	String ext = "";
    	int inx = filename.lastIndexOf(".");
    	if (inx > 0) {
    		ext = filename.substring(inx);
    	}
    	return ext;
    }

}
