package org.playframework.playtools.util.invoker;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.Launch;
import org.eclipse.jdt.launching.IVMRunner;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
import org.playframework.playtools.util.PlayToolsUtil;
import org.playframework.playtools.util.console.PlayMessageConsole;

public class PlayInvokerVersion1 extends AbstractPlayInvoker {

	private static ILaunch launch = null;

	public PlayInvokerVersion1() {
	}
	
	public PlayInvokerVersion1(String payRootPath, String version, String projectPath, String port) {
		this.payRootPath = payRootPath;
		this.version = version;
		this.projectPath = projectPath;
		this.port = port;
		
		consoleTitle = "play framework";
	}
	
	public void stopPlay() {
		if (launch != null) {
			if (messageConsole != null) {
				messageConsole.println("\n" + consoleTitle + " version " + version + " stop...");
			}
			try {
				launch.terminate();
			} catch (DebugException e) {
				e.printStackTrace();
			}
			DebugPlugin.getDefault().getLaunchManager().removeLaunch(launch);
			launch = null;
		}
	}
	
	protected void launchPlay() {

		messageConsole = new PlayMessageConsole(consoleTitle);

		List<String> listClass = PlayToolsUtil.getClasspath(payRootPath);
		listClass.add(projectPath + "/conf");
		
		List<String> argList = new ArrayList<String>();

		argList.add("-XX:-UseSplitVerifier");
		argList.add("-Dfile.encoding=utf-8");
		argList.add("-Xrunjdwp:transport=dt_socket,address=" + port + ",server=y,suspend=n");
		argList.add("-Xdebug");
		argList.add("-Dplay.debug=yes");
		argList.add("-Dapplication.path=" + projectPath);

		messageConsole.println(consoleTitle + " version " + version + " start...");

        VMRunnerConfiguration vmConfig = new VMRunnerConfiguration("play.server.Server", (String[])listClass.toArray(new String[listClass.size()]));

		vmConfig.setVMArguments((String[])argList.toArray(new String[argList.size()]));

		IVMRunner vmRunner = JavaRuntime.getDefaultVMInstall().getVMRunner(ILaunchManager.RUN_MODE);

		launch = new Launch(null, ILaunchManager.RUN_MODE, null);

		try {
			vmRunner.run(vmConfig, launch, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}

		DebugPlugin.getDefault().getLaunchManager().addLaunch(launch);

        launch.getProcesses()[0].getStreamsProxy().getOutputStreamMonitor().addListener(messageConsole.getListener());
        launch.getProcesses()[0].getStreamsProxy().getErrorStreamMonitor().addListener(messageConsole.getListener());

	}

}
